@@ -326,7 +326,7 @@ class ImportPathsTest(_LoaderTest):
         from . import path as path
         _PathType = path._PathType
         def utime(path: _PathType) -> None: ...
-        class stat_result(object): ...
+        class stat_result: ...
       """)
       d.create_file("os2/path.pyi", """
         import os2
@@ -391,20 +391,6 @@ class ImportPathsTest(_LoaderTest):
       x = ast.Lookup("test.x")
       self.assertIsInstance(x.type, pytd.IntersectionType)
 
-  def test_python2_builtins(self):
-    # Test that we read python2 builtins from builtin.pytd if we pass a python2
-    # version to the loader.
-    with file_utils.Tempdir() as d:
-      d.create_file("a.pyi", """
-          from UserDict import UserDict
-          class A(UserDict): ...""")
-      loader = load_pytd.Loader("base",
-                                python_version=(2, 7),
-                                pythonpath=[d.path])
-      a = loader.import_name("a")
-      cls = a.Lookup("a.A")
-      self.assertEqual("UserDict.UserDict", pytd_utils.Print(cls.parents[0]))
-
   def test_open_function(self):
     def mock_open(*unused_args, **unused_kwargs):
       return io.StringIO("x: int")
@@ -483,6 +469,16 @@ class ImportPathsTest(_LoaderTest):
       from b import *
     """)
 
+  def test_import_class_from_parent_module(self):
+    with file_utils.Tempdir() as d:
+      d.create_file("foo/__init__.pyi", "class Foo: ...")
+      d.create_file("foo/bar.pyi", """
+        from . import Foo
+        class Bar(Foo): ...
+      """)
+      loader = load_pytd.Loader(None, self.python_version, pythonpath=[d.path])
+      loader.import_name("foo.bar")
+
 
 class ImportTypeMacroTest(_LoaderTest):
 
@@ -552,7 +548,7 @@ class PickledPyiLoaderTest(test_base.UnitTest):
         x = List[int]
         b = List[int]
 
-        class SomeClass(object):
+        class SomeClass:
           def __init__(self, a: module2.ObjectMod2):
             pass
 
@@ -561,7 +557,7 @@ class PickledPyiLoaderTest(test_base.UnitTest):
     """
     tempdir.create_file("module1.pyi", src)
     tempdir.create_file("module2.pyi", """
-        class ObjectMod2(object):
+        class ObjectMod2:
           def __init__(self):
             pass
     """)
@@ -608,7 +604,7 @@ class PickledPyiLoaderTest(test_base.UnitTest):
 
   def test_star_import(self):
     with file_utils.Tempdir() as d:
-      d.create_file("foo.pyi", "class A(object): ...")
+      d.create_file("foo.pyi", "class A: ...")
       d.create_file("bar.pyi", "from foo import *")
       foo = _Module(module_name="foo", file_name="foo.pyi")
       bar = _Module(module_name="bar", file_name="bar.pyi")
