@@ -17,8 +17,8 @@ log = logging.getLogger(__name__)
 class AddMetaclassInstance(abstract.BaseValue):
   """AddMetaclass instance (constructed by AddMetaclass.call())."""
 
-  def __init__(self, meta, vm, module_name):
-    super().__init__("AddMetaclassInstance", vm)
+  def __init__(self, meta, ctx, module_name):
+    super().__init__("AddMetaclassInstance", ctx)
     self.meta = meta
     self.module_name = module_name
 
@@ -26,7 +26,7 @@ class AddMetaclassInstance(abstract.BaseValue):
     if len(args.posargs) != 1:
       sig = function.Signature.from_param_names(
           "%s.add_metaclass" % self.module_name, ("cls",))
-      raise function.WrongArgCount(sig, args, self.vm)
+      raise function.WrongArgCount(sig, args, self.ctx)
     cls_var = args.posargs[0]
     for b in cls_var.bindings:
       cls = b.data
@@ -44,8 +44,8 @@ class AddMetaclass(abstract.PyTDFunction):
   """Implements the add_metaclass decorator."""
 
   @classmethod
-  def make(cls, name, vm, module_name):
-    self = super().make(name, vm, module_name)
+  def make(cls, name, ctx, module_name):
+    self = super().make(name, ctx, module_name)
     self.module_name = module_name
     return self
 
@@ -53,16 +53,16 @@ class AddMetaclass(abstract.PyTDFunction):
     """Adds a metaclass."""
     self.match_args(node, args)
     meta = abstract_utils.get_atomic_value(
-        args.posargs[0], default=self.vm.convert.unsolvable)
-    return node, AddMetaclassInstance(
-        meta, self.vm, self.module_name).to_variable(node)
+        args.posargs[0], default=self.ctx.convert.unsolvable)
+    return node, AddMetaclassInstance(meta, self.ctx,
+                                      self.module_name).to_variable(node)
 
 
 class WithMetaclassInstance(abstract.BaseValue, class_mixin.Class):
   """Anonymous class created by with_metaclass."""
 
-  def __init__(self, vm, cls, bases):
-    super().__init__("WithMetaclassInstance", vm)
+  def __init__(self, ctx, cls, bases):
+    super().__init__("WithMetaclassInstance", ctx)
     class_mixin.Class.init_mixin(self, cls)
     self.bases = bases
 
@@ -86,7 +86,7 @@ class WithMetaclass(abstract.PyTDFunction):
     """Creates an anonymous class to act as a metaclass."""
     self.match_args(node, args)
     meta = abstract_utils.get_atomic_value(
-        args.posargs[0], default=self.vm.convert.unsolvable)
+        args.posargs[0], default=self.ctx.convert.unsolvable)
     bases = args.posargs[1:]
-    result = WithMetaclassInstance(self.vm, meta, bases).to_variable(node)
+    result = WithMetaclassInstance(self.ctx, meta, bases).to_variable(node)
     return node, result
