@@ -7,35 +7,35 @@ from pytype import overlay
 class TypesOverlay(overlay.Overlay):
   """A custom overlay for the 'types' module."""
 
-  def __init__(self, vm):
+  def __init__(self, ctx):
     member_map = {
         "coroutine": CoroutineDecorator.make_for_types
     }
-    ast = vm.loader.import_name("types")
-    super().__init__(vm, "types", member_map, ast)
+    ast = ctx.loader.import_name("types")
+    super().__init__(ctx, "types", member_map, ast)
 
 
 class AsyncioOverlay(overlay.Overlay):
   """A custom overlay for the 'asyncio' module."""
 
-  def __init__(self, vm):
+  def __init__(self, ctx):
     member_map = {
         "coroutine": CoroutineDecorator.make_for_asyncio
     }
-    ast = vm.loader.import_name("asyncio")
-    super().__init__(vm, "asyncio", member_map, ast)
+    ast = ctx.loader.import_name("asyncio")
+    super().__init__(ctx, "asyncio", member_map, ast)
 
 
 class CoroutineDecorator(abstract.PyTDFunction):
   """Implements the @types.coroutine and @asyncio.coroutine decorator."""
 
   @classmethod
-  def make_for_types(cls, vm):
-    return super().make("coroutine", vm, "types")
+  def make_for_types(cls, ctx):
+    return super().make("coroutine", ctx, "types")
 
   @classmethod
-  def make_for_asyncio(cls, vm):
-    return super().make("coroutine", vm, "asyncio")
+  def make_for_asyncio(cls, ctx):
+    return super().make("coroutine", ctx, "asyncio")
 
   def call(self, node, unused_func, args):
     """Marks the function as a generator-based coroutine."""
