@@ -28,50 +28,50 @@ Param = overlay_utils.Param
 class TypingOverlay(overlay.Overlay):
   """A representation of the 'typing' module that allows custom overlays."""
 
-  def __init__(self, vm):
+  def __init__(self, ctx):
     # Make sure we have typing available as a dependency
     member_map = typing_overlay.copy()
-    ast = vm.loader.typing
+    ast = ctx.loader.typing
     for cls in ast.classes:
       _, name = cls.name.rsplit(".", 1)
       if name not in member_map and pytd.IsContainer(cls) and cls.template:
         member_map[name] = overlay.build(name, TypingContainer)
-    super().__init__(vm, "typing", member_map, ast)
+    super().__init__(ctx, "typing", member_map, ast)
 
 
 class Union(abstract.AnnotationClass):
   """Implementation of typing.Union[...]."""
 
-  def __init__(self, vm, options=()):
-    super().__init__("Union", vm)
+  def __init__(self, ctx, options=()):
+    super().__init__("Union", ctx)
     self.options = options
 
   def _build_value(self, node, inner, ellipses):
-    self.vm.errorlog.invalid_ellipses(self.vm.frames, ellipses, self.name)
-    return abstract.Union(self.options + inner, self.vm)
+    self.ctx.errorlog.invalid_ellipses(self.ctx.vm.frames, ellipses, self.name)
+    return abstract.Union(self.options + inner, self.ctx)
 
 
 class Annotated(abstract.AnnotationClass):
   """Implementation of typing.Annotated[T, *annotations]."""
 
   def _build_value(self, node, inner, ellipses):
-    self.vm.errorlog.invalid_ellipses(self.vm.frames, ellipses, self.name)
+    self.ctx.errorlog.invalid_ellipses(self.ctx.vm.frames, ellipses, self.name)
     if len(inner) == 1:
       error = "typing.Annotated must have at least 1 annotation"
-      self.vm.errorlog.invalid_annotation(self.vm.frames, self, error)
+      self.ctx.errorlog.invalid_annotation(self.ctx.vm.frames, self, error)
     # discard annotations
     return inner[0]
 
 
 class TypingContainer(abstract.AnnotationContainer):
 
-  def __init__(self, name, vm):
+  def __init__(self, name, ctx):
     if name in pep484.PEP484_CAPITALIZED:
       pytd_name = "builtins." + name.lower()
     else:
       pytd_name = "typing." + name
-    base = vm.convert.name_to_value(pytd_name)
-    super().__init__(name, vm, base)
+    base = ctx.convert.name_to_value(pytd_name)
+    super().__init__(name, ctx, base)
 
 
 class Tuple(TypingContainer):
@@ -85,7 +85,7 @@ class Tuple(TypingContainer):
           inner, ellipses, allowed_ellipses={len(inner) - 1} - {0})
     else:
       template = list(range(len(inner))) + [abstract_utils.T]
-      inner += (self.vm.merge_values(inner),)
+      inner += (self.ctx.convert.merge_values(inner),)
       return template, inner, abstract.TupleClass
 
 
@@ -98,16 +98,17 @@ class Callable(TypingContainer):
     args = inner[0]
     if abstract_utils.is_concrete_list(args):
       inner[0], inner_ellipses = self._build_inner(args.pyval)
-      self.vm.errorlog.invalid_ellipses(
-          self.vm.frames, inner_ellipses, args.name)
+      self.ctx.errorlog.invalid_ellipses(self.ctx.vm.frames, inner_ellipses,
+                                         args.name)
     else:
       if args.cls.full_name == "builtins.list":
-        self.vm.errorlog.ambiguous_annotation(self.vm.frames, [args])
+        self.ctx.errorlog.ambiguous_annotation(self.ctx.vm.frames, [args])
       elif 0 not in ellipses or not isinstance(args, abstract.Unsolvable):
-        self.vm.errorlog.invalid_annotation(
-            self.vm.frames, args, ("First argument to Callable must be a list"
-                                   " of argument types or ellipsis."))
-      inner[0] = self.vm.convert.unsolvable
+        self.ctx.errorlog.invalid_annotation(
+            self.ctx.vm.frames, args,
+            ("First argument to Callable must be a list"
+             " of argument types or ellipsis."))
+      inner[0] = self.ctx.convert.unsolvable
     value = self._build_value(node, tuple(inner), ellipses)
     return node, value.to_variable(node)
 
@@ -115,9 +116,10 @@ class Callable(TypingContainer):
     if isinstance(inner[0], list):
       template = (list(range(len(inner[0]))) +
                   [t.name for t in self.base_cls.template])
-      combined_args = self.vm.merge_values(inner[0])
+      combined_args = self.ctx.convert.merge_values(inner[0])
       inner = tuple(inner[0]) + (combined_args,) + inner[1:]
-      self.vm.errorlog.invalid_ellipses(self.vm.frames, ellipses, self.name)
+      self.ctx.errorlog.invalid_ellipses(self.ctx.vm.frames, ellipses,
+                                         self.name)
       return template, inner, abstract.CallableClass
     else:
       # An ellipsis may take the place of the ARGS list.
@@ -144,9 +146,9 @@ class TypeVar(abstract.PyTDFunction):
     return ret
 
   def _get_annotation(self, node, var, name):
-    with self.vm.errorlog.checkpoint() as record:
-      annot = self.vm.annotation_utils.extract_annotation(
-          node, var, name, self.vm.simple_stack())
+    with self.ctx.errorlog.checkpoint() as record:
+      annot = self.ctx.annotation_utils.extract_annotation(
+          node, var, name, self.ctx.vm.simple_stack())
     if record.errors:
       raise TypeVarError("\n".join(error.message for error in record.errors))
     return annot
@@ -159,12 +161,12 @@ class TypeVar(abstract.PyTDFunction):
     else:
       ret = self._get_constant(args.namedargs[name], name, bool)
       # This error is logged only if _get_constant succeeds.
-      self.vm.errorlog.not_supported_yet(
-          self.vm.frames, f"argument \"{name}\" to TypeVar")
+      self.ctx.errorlog.not_supported_yet(self.ctx.vm.frames,
+                                          f"argument \"{name}\" to TypeVar")
       return ret
 
   def _get_typeparam(self, node, args):
-    args = args.simplify(node, self.vm)
+    args = args.simplify(node, self.ctx)
     try:
       self.match_args(node, args)
     except function.InvalidParameters as e:
@@ -191,18 +193,22 @@ class TypeVar(abstract.PyTDFunction):
       raise TypeVarError("*args must be a constant tuple")
     if args.starstarargs:
       raise TypeVarError("ambiguous **kwargs not allowed")
-    return abstract.TypeParameter(name, self.vm, constraints=constraints,
-                                  bound=bound, covariant=covariant,
-                                  contravariant=contravariant)
+    return abstract.TypeParameter(
+        name,
+        self.ctx,
+        constraints=constraints,
+        bound=bound,
+        covariant=covariant,
+        contravariant=contravariant)
 
   def call(self, node, _, args):
     """Call typing.TypeVar()."""
     try:
       param = self._get_typeparam(node, args)
     except TypeVarError as e:
-      self.vm.errorlog.invalid_typevar(
-          self.vm.frames, utils.message(e), e.bad_call)
-      return node, self.vm.new_unsolvable(node)
+      self.ctx.errorlog.invalid_typevar(self.ctx.vm.frames, utils.message(e),
+                                        e.bad_call)
+      return node, self.ctx.new_unsolvable(node)
     return node, param.to_variable(node)
 
 
@@ -211,7 +217,7 @@ class Cast(abstract.PyTDFunction):
 
   def call(self, node, func, args):
     if args.posargs:
-      _, value = self.vm.annotation_utils.extract_and_init_annotation(
+      _, value = self.ctx.annotation_utils.extract_and_init_annotation(
           node, "typing.cast", args.posargs[0])
       return node, value
     return super().call(node, func, args)
@@ -220,32 +226,32 @@ class Cast(abstract.PyTDFunction):
 class NoReturn(abstract.Singleton):
   """Implements typing.NoReturn as a singleton."""
 
-  def __init__(self, vm):
-    super().__init__("NoReturn", vm)
+  def __init__(self, ctx):
+    super().__init__("NoReturn", ctx)
 
 
-def build_any(vm):
-  return vm.convert.unsolvable
+def build_any(ctx):
+  return ctx.convert.unsolvable
 
 
 class NamedTupleFuncBuilder(collections_overlay.NamedTupleBuilder):
   """Factory for creating typing.NamedTuple classes."""
 
   @classmethod
-  def make(cls, vm):
-    typing_ast = vm.loader.import_name("typing")
+  def make(cls, ctx):
+    typing_ast = ctx.loader.import_name("typing")
     # Because NamedTuple is a special case for the pyi parser, typing.pytd has
     # "_NamedTuple" instead. Replace the name of the returned function so that
     # error messages will correctly display "typing.NamedTuple".
     pyval = typing_ast.Lookup("typing._NamedTuple")
     pyval = pyval.Replace(name="typing.NamedTuple")
-    self = super().make("NamedTuple", vm, pyval)
+    self = super().make("NamedTuple", ctx, pyval)
     # NamedTuple's fields arg has type Sequence[Sequence[Union[str, type]]],
     # which doesn't provide precise enough type-checking, so we have to do
     # some of our own in _getargs. _NamedTupleFields is an alias to
     # List[Tuple[str, type]], which gives a more understandable error message.
     fields_pyval = typing_ast.Lookup("typing._NamedTupleFields").type
-    fields_type = vm.convert.constant_to_value(fields_pyval, {}, vm.root_node)
+    fields_type = ctx.convert.constant_to_value(fields_pyval, {}, ctx.root_node)
     # pylint: disable=protected-access
     self._fields_param = function.BadParam(name="fields", expected=fields_type)
     return self
@@ -264,8 +270,8 @@ class NamedTupleFuncBuilder(collections_overlay.NamedTupleBuilder):
     fields = abstract_utils.get_atomic_python_constant(fields_var)
     if isinstance(fields, str):
       # Since str matches Sequence, we have to manually check for it.
-      raise function.WrongArgTypes(
-          sig.signature, args, self.vm, self._fields_param)
+      raise function.WrongArgTypes(sig.signature, args, self.ctx,
+                                   self._fields_param)
     # The fields is a list of tuples, so we need to deeply unwrap them.
     fields = [abstract_utils.get_atomic_python_constant(t) for t in fields]
     # We need the actual string for the field names and the BaseValue
@@ -275,22 +281,25 @@ class NamedTupleFuncBuilder(collections_overlay.NamedTupleBuilder):
     for field in fields:
       if isinstance(field, str):
         # Since str matches Sequence, we have to manually check for it.
-        raise function.WrongArgTypes(
-            sig.signature, args, self.vm, self._fields_param)
+        raise function.WrongArgTypes(sig.signature, args, self.ctx,
+                                     self._fields_param)
       if (len(field) != 2 or
           any(not self._is_str_instance(v) for v in field[0].data)):
         # Note that we don't need to check field[1] because both 'str'
         # (forward reference) and 'type' are valid for it.
-        raise function.WrongArgTypes(
-            sig.signature, args, self.vm, self._fields_param)
+        raise function.WrongArgTypes(sig.signature, args, self.ctx,
+                                     self._fields_param)
       name, typ = field
       name_py_constant = abstract_utils.get_atomic_python_constant(name)
       names.append(name_py_constant)
       if functional:
         allowed_type_params = (
-            self.vm.annotation_utils.get_callable_type_parameter_names(typ))
-        annot = self.vm.annotation_utils.extract_annotation(
-            node, typ, name_py_constant, self.vm.simple_stack(),
+            self.ctx.annotation_utils.get_callable_type_parameter_names(typ))
+        annot = self.ctx.annotation_utils.extract_annotation(
+            node,
+            typ,
+            name_py_constant,
+            self.ctx.vm.simple_stack(),
             allowed_type_params=allowed_type_params)
       else:
         # This NamedTuple was constructed with the class syntax. The field
@@ -303,43 +312,38 @@ class NamedTupleFuncBuilder(collections_overlay.NamedTupleBuilder):
     # Build an InterpreterClass representing the namedtuple.
     if field_types:
       # TODO(mdemello): Fix this to support late types.
-      field_types_union = abstract.Union(field_types, self.vm)
+      field_types_union = abstract.Union(field_types, self.ctx)
     else:
-      field_types_union = self.vm.convert.none_type
+      field_types_union = self.ctx.convert.none_type
 
     members = {n: t.instantiate(node) for n, t in zip(field_names, field_types)}
 
     # collections.namedtuple has: __dict__, __slots__ and _fields.
     # typing.NamedTuple adds: _field_types, __annotations__ and _field_defaults.
     # __slots__ and _fields are tuples containing the names of the fields.
-    slots = tuple(self.vm.convert.build_string(node, f) for f in field_names)
-    members["__slots__"] = abstract.Tuple(slots, self.vm).to_variable(node)
-    members["_fields"] = abstract.Tuple(slots, self.vm).to_variable(node)
+    slots = tuple(self.ctx.convert.build_string(node, f) for f in field_names)
+    members["__slots__"] = abstract.Tuple(slots, self.ctx).to_variable(node)
+    members["_fields"] = abstract.Tuple(slots, self.ctx).to_variable(node)
     # __dict__ and _field_defaults are both collections.OrderedDicts that map
     # field names (strings) to objects of the field types.
-    ordered_dict_cls = self.vm.convert.name_to_value("collections.OrderedDict",
-                                                     ast=self.collections_ast)
-
-    # In Python 2, keys can be `str` or `unicode`; support both.
-    # In Python 3, `str_type` and `unicode_type` are the same.
-    field_keys_union = abstract.Union([self.vm.convert.str_type,
-                                       self.vm.convert.unicode_type], self.vm)
+    ordered_dict_cls = self.ctx.convert.name_to_value(
+        "collections.OrderedDict", ast=self.collections_ast)
 
     # Normally, we would use abstract_utils.K and abstract_utils.V, but
     # collections.pyi doesn't conform to that standard.
-    field_dict_cls = abstract.ParameterizedClass(
-        ordered_dict_cls,
-        {"K": field_keys_union, "V": field_types_union},
-        self.vm)
+    field_dict_cls = abstract.ParameterizedClass(ordered_dict_cls, {
+        "K": self.ctx.convert.str_type,
+        "V": field_types_union
+    }, self.ctx)
     members["__dict__"] = field_dict_cls.instantiate(node)
     members["_field_defaults"] = field_dict_cls.instantiate(node)
     # _field_types and __annotations__ are both collections.OrderedDicts
     # that map field names (strings) to the types of the fields. Note that
     # vm.make_class will take care of adding the __annotations__ member.
-    field_types_cls = abstract.ParameterizedClass(
-        ordered_dict_cls,
-        {"K": field_keys_union, "V": self.vm.convert.type_type},
-        self.vm)
+    field_types_cls = abstract.ParameterizedClass(ordered_dict_cls, {
+        "K": self.ctx.convert.str_type,
+        "V": self.ctx.convert.type_type
+    }, self.ctx)
     members["_field_types"] = field_types_cls.instantiate(node)
 
     # __new__
@@ -350,12 +354,15 @@ class NamedTupleFuncBuilder(collections_overlay.NamedTupleBuilder):
     # their constructors.
     cls_type_param = abstract.TypeParameter(
         visitors.CreateTypeParametersForSignatures.PREFIX + name,
-        self.vm, bound=None)
-    cls_type = abstract.ParameterizedClass(
-        self.vm.convert.type_type, {abstract_utils.T: cls_type_param}, self.vm)
+        self.ctx,
+        bound=None)
+    cls_type = abstract.ParameterizedClass(self.ctx.convert.type_type,
+                                           {abstract_utils.T: cls_type_param},
+                                           self.ctx)
     params = [Param(n, t) for n, t in zip(field_names, field_types)]
     members["__new__"] = overlay_utils.make_method(
-        self.vm, node,
+        self.ctx,
+        node,
         name="__new__",
         self_param=Param("cls", cls_type),
         params=params,
@@ -364,7 +371,8 @@ class NamedTupleFuncBuilder(collections_overlay.NamedTupleBuilder):
 
     # __init__
     members["__init__"] = overlay_utils.make_method(
-        self.vm, node,
+        self.ctx,
+        node,
         name="__init__",
         varargs=Param("args"),
         kwargs=Param("kwargs"))
@@ -373,38 +381,41 @@ class NamedTupleFuncBuilder(collections_overlay.NamedTupleBuilder):
     # _make is a classmethod, so it needs to be wrapped by
     # specialibuiltins.ClassMethodInstance.
     # Like __new__, it uses the _Tname TypeVar.
-    sized_cls = self.vm.convert.name_to_value("typing.Sized")
+    sized_cls = self.ctx.convert.name_to_value("typing.Sized")
     iterable_type = abstract.ParameterizedClass(
-        self.vm.convert.name_to_value("typing.Iterable"),
-        {abstract_utils.T: field_types_union}, self.vm)
-    cls_type = abstract.ParameterizedClass(
-        self.vm.convert.type_type,
-        {abstract_utils.T: cls_type_param}, self.vm)
+        self.ctx.convert.name_to_value("typing.Iterable"),
+        {abstract_utils.T: field_types_union}, self.ctx)
+    cls_type = abstract.ParameterizedClass(self.ctx.convert.type_type,
+                                           {abstract_utils.T: cls_type_param},
+                                           self.ctx)
     len_type = abstract.CallableClass(
-        self.vm.convert.name_to_value("typing.Callable"),
-        {0: sized_cls,
-         abstract_utils.ARGS: sized_cls,
-         abstract_utils.RET: self.vm.convert.int_type},
-        self.vm)
+        self.ctx.convert.name_to_value("typing.Callable"), {
+            0: sized_cls,
+            abstract_utils.ARGS: sized_cls,
+            abstract_utils.RET: self.ctx.convert.int_type
+        }, self.ctx)
     params = [
         Param("iterable", iterable_type),
-        Param("new").unsolvable(self.vm, node),
-        Param("len", len_type).unsolvable(self.vm, node)]
+        Param("new").unsolvable(self.ctx, node),
+        Param("len", len_type).unsolvable(self.ctx, node)
+    ]
     make = overlay_utils.make_method(
-        self.vm, node,
+        self.ctx,
+        node,
         name="_make",
         params=params,
         self_param=Param("cls", cls_type),
         return_type=cls_type_param)
     make_args = function.Args(posargs=(make,))
-    _, members["_make"] = self.vm.special_builtins["classmethod"].call(
+    _, members["_make"] = self.ctx.special_builtins["classmethod"].call(
         node, None, make_args)
 
     # _replace
     # Like __new__, it uses the _Tname TypeVar. We have to annotate the `self`
     # param to make sure the TypeVar is substituted correctly.
     members["_replace"] = overlay_utils.make_method(
-        self.vm, node,
+        self.ctx,
+        node,
         name="_replace",
         self_param=Param("self", cls_type_param),
         return_type=cls_type_param,
@@ -414,47 +425,43 @@ class NamedTupleFuncBuilder(collections_overlay.NamedTupleBuilder):
     getnewargs_tuple_params = dict(
         tuple(enumerate(field_types)) +
         ((abstract_utils.T, field_types_union),))
-    getnewargs_tuple = abstract.TupleClass(self.vm.convert.tuple_type,
-                                           getnewargs_tuple_params, self.vm)
+    getnewargs_tuple = abstract.TupleClass(self.ctx.convert.tuple_type,
+                                           getnewargs_tuple_params, self.ctx)
     members["__getnewargs__"] = overlay_utils.make_method(
-        self.vm, node,
-        name="__getnewargs__",
-        return_type=getnewargs_tuple)
+        self.ctx, node, name="__getnewargs__", return_type=getnewargs_tuple)
 
     # __getstate__
     members["__getstate__"] = overlay_utils.make_method(
-        self.vm, node, name="__getstate__")
+        self.ctx, node, name="__getstate__")
 
     # _asdict
     members["_asdict"] = overlay_utils.make_method(
-        self.vm, node,
-        name="_asdict",
-        return_type=field_dict_cls)
+        self.ctx, node, name="_asdict", return_type=field_dict_cls)
 
     # Finally, make the class.
-    cls_dict = abstract.Dict(self.vm)
+    cls_dict = abstract.Dict(self.ctx)
     cls_dict.update(node, members)
 
     if bases:
       final_bases = []
       for base in bases:
         if any(b.full_name == "typing.NamedTuple" for b in base.data):
-          final_bases.append(self.vm.convert.tuple_type.to_variable(node))
+          final_bases.append(self.ctx.convert.tuple_type.to_variable(node))
         else:
           final_bases.append(base)
     else:
-      final_bases = [self.vm.convert.tuple_type.to_variable(node)]
+      final_bases = [self.ctx.convert.tuple_type.to_variable(node)]
       # This NamedTuple is being created via a function call. We manually
       # construct an annotated_locals entry for it so that __annotations__ is
       # initialized properly for the generated class.
-      self.vm.annotated_locals[name] = {
-          field: abstract_utils.Local(node, None, typ, None, self.vm)
+      self.ctx.vm.annotated_locals[name] = {
+          field: abstract_utils.Local(node, None, typ, None, self.ctx)
           for field, typ in zip(field_names, field_types)
       }
 
-    node, cls_var = self.vm.make_class(
+    node, cls_var = self.ctx.vm.make_class(
         node=node,
-        name_var=self.vm.convert.build_string(node, name),
+        name_var=self.ctx.convert.build_string(node, name),
         bases=final_bases,
         class_dict_var=cls_dict.to_variable(node),
         cls_var=None)
@@ -471,27 +478,30 @@ class NamedTupleFuncBuilder(collections_overlay.NamedTupleBuilder):
       name_var, field_names, field_types = self._getargs(
           node, args, functional=bases is None)
     except abstract_utils.ConversionError:
-      return node, self.vm.new_unsolvable(node)
+      return node, self.ctx.new_unsolvable(node)
 
     try:
       name = abstract_utils.get_atomic_python_constant(name_var)
     except abstract_utils.ConversionError:
-      return node, self.vm.new_unsolvable(node)
+      return node, self.ctx.new_unsolvable(node)
 
     try:
       field_names = self._validate_and_rename_args(name, field_names, False)
     except ValueError as e:
-      self.vm.errorlog.invalid_namedtuple_arg(self.vm.frames, utils.message(e))
-      return node, self.vm.new_unsolvable(node)
+      self.ctx.errorlog.invalid_namedtuple_arg(self.ctx.vm.frames,
+                                               utils.message(e))
+      return node, self.ctx.new_unsolvable(node)
 
-    annots = self.vm.annotation_utils.convert_annotations_list(
+    annots = self.ctx.annotation_utils.convert_annotations_list(
         node, zip(field_names, field_types))
-    field_types = [annots.get(field_name, self.vm.convert.unsolvable)
-                   for field_name in field_names]
+    field_types = [
+        annots.get(field_name, self.ctx.convert.unsolvable)
+        for field_name in field_names
+    ]
     node, cls_var = self._build_namedtuple(
         name, field_names, field_types, node, bases)
 
-    self.vm.trace_classdef(cls_var)
+    self.ctx.vm.trace_classdef(cls_var)
     return node, cls_var
 
 
@@ -503,37 +513,41 @@ class NamedTupleClassBuilder(abstract.PyTDClass):
                  "_fields", "_field_defaults", "_field_types",
                  "_make", "_replace", "_asdict", "_source")
 
-  def __init__(self, vm):
-    typing_ast = vm.loader.import_name("typing")
+  def __init__(self, ctx):
+    typing_ast = ctx.loader.import_name("typing")
     pyval = typing_ast.Lookup("typing._NamedTupleClass")
     pyval = pyval.Replace(name="typing.NamedTuple")
-    super().__init__("NamedTuple", pyval, vm)
+    super().__init__("NamedTuple", pyval, ctx)
     # Prior to python 3.6, NamedTuple is a function. Although NamedTuple is a
     # class in python 3.6+, we can still use it like a function. Hold the
     # an instance of 'NamedTupleFuncBuilder' so that we can reuse the
     # old implementation to implement the NamedTuple in python 3.6+
-    self.namedtuple = NamedTupleFuncBuilder.make(vm)
+    self.namedtuple = NamedTupleFuncBuilder.make(ctx)
 
   def call(self, node, _, args):
     posargs = args.posargs
     if isinstance(args.namedargs, dict):
       namedargs = args.namedargs
     else:
-      namedargs = self.vm.convert.value_to_constant(args.namedargs, dict)
-    if namedargs and self.vm.python_version < (3, 6):
+      namedargs = self.ctx.convert.value_to_constant(args.namedargs, dict)
+    if namedargs and self.ctx.python_version < (3, 6):
       errmsg = "Keyword syntax for NamedTuple is only supported in Python 3.6+"
-      self.vm.errorlog.invalid_namedtuple_arg(self.vm.frames, err_msg=errmsg)
+      self.ctx.errorlog.invalid_namedtuple_arg(
+          self.ctx.vm.frames, err_msg=errmsg)
     if namedargs and len(posargs) == 1:
-      namedargs = [abstract.Tuple(
-          (self.vm.convert.build_string(node, k), v), self.vm).to_variable(node)
-                   for k, v in namedargs.items()]
-      namedargs = abstract.List(namedargs, self.vm).to_variable(node)
+      namedargs = [
+          abstract.Tuple((self.ctx.convert.build_string(node, k), v),
+                         self.ctx).to_variable(node)
+          for k, v in namedargs.items()
+      ]
+      namedargs = abstract.List(namedargs, self.ctx).to_variable(node)
       posargs += (namedargs,)
       args = function.Args(posargs)
     elif namedargs:
       errmsg = ("Either list of fields or keywords can be provided to "
                 "NamedTuple, not both")
-      self.vm.errorlog.invalid_namedtuple_arg(self.vm.frames, err_msg=errmsg)
+      self.ctx.errorlog.invalid_namedtuple_arg(
+          self.ctx.vm.frames, err_msg=errmsg)
     return self.namedtuple.call(node, None, args)
 
   def make_class(self, node, bases, f_locals):
@@ -541,8 +555,8 @@ class NamedTupleClassBuilder(abstract.PyTDClass):
     # Since we don't support defining NamedTuple subclasses in a nested scope
     # anyway, we can just return unsolvable here to prevent a crash, and let the
     # invalid namedtuple error get raised later.
-    if f_locals.data[0].isinstance_Unsolvable():
-      return node, self.vm.new_unsolvable(node)
+    if isinstance(f_locals.data[0], abstract.Unsolvable):
+      return node, self.ctx.new_unsolvable(node)
 
     f_locals = abstract_utils.get_atomic_python_constant(f_locals)
 
@@ -551,21 +565,23 @@ class NamedTupleClassBuilder(abstract.PyTDClass):
     nameval = abstract_utils.get_atomic_python_constant(name)
     if "." in nameval:
       nameval = nameval.rsplit(".", 1)[-1]
-      name = self.vm.convert.constant_to_var(nameval)
+      name = self.ctx.convert.constant_to_var(nameval)
 
     # assemble the arguments that are compatible with NamedTupleFuncBuilder.call
     field_list = []
     defaults = []
     cls_locals = classgen.get_class_locals(
-        nameval, allow_methods=True, ordering=classgen.Ordering.FIRST_ANNOTATE,
-        vm=self.vm)
+        nameval,
+        allow_methods=True,
+        ordering=classgen.Ordering.FIRST_ANNOTATE,
+        ctx=self.ctx)
     for k, local in cls_locals.items():
       assert local.typ
       if k in f_locals:
         defaults.append(f_locals[k])
-      k = self.vm.convert.constant_to_var(k, node=node)
-      field_list.append(self.vm.convert.build_tuple(node, (k, local.typ)))
-    anno = self.vm.convert.build_list(node, field_list)
+      k = self.ctx.convert.constant_to_var(k, node=node)
+      field_list.append(self.ctx.convert.build_tuple(node, (k, local.typ)))
+    anno = self.ctx.convert.build_list(node, field_list)
     posargs = (name, anno)
     args = function.Args(posargs=posargs)
     node, cls_var = self.namedtuple.call(node, None, args, bases)
@@ -573,24 +589,24 @@ class NamedTupleClassBuilder(abstract.PyTDClass):
 
     if not isinstance(cls_val, abstract.Unsolvable):
       # set __new__.__defaults__
-      defaults = abstract.Tuple(tuple(defaults), self.vm).to_variable(node)
-      node, new_attr = self.vm.attribute_handler.get_attribute(
+      defaults = abstract.Tuple(tuple(defaults), self.ctx).to_variable(node)
+      node, new_attr = self.ctx.attribute_handler.get_attribute(
           node, cls_val, "__new__")
       new_attr = abstract_utils.get_atomic_value(new_attr)
-      node = self.vm.attribute_handler.set_attribute(
-          node, new_attr, "__defaults__", defaults)
+      node = self.ctx.attribute_handler.set_attribute(node, new_attr,
+                                                      "__defaults__", defaults)
 
       # set the attribute without overriding special namedtuple attributes
-      node, fields = self.vm.attribute_handler.get_attribute(
+      node, fields = self.ctx.attribute_handler.get_attribute(
           node, cls_val, "_fields")
       fields = abstract_utils.get_atomic_python_constant(fields, tuple)
       fields = [abstract_utils.get_atomic_python_constant(field, str)
                 for field in fields]
       for key in f_locals:
         if key in self._prohibited:
-          self.vm.errorlog.not_writable(self.vm.frames, cls_val, key)
+          self.ctx.errorlog.not_writable(self.ctx.vm.frames, cls_val, key)
         if key not in abstract_utils.CLASS_LEVEL_IGNORE and  key not in fields:
-          node = self.vm.attribute_handler.set_attribute(
+          node = self.ctx.attribute_handler.set_attribute(
               node, cls_val, key, f_locals[key])
 
     return node, cls_var
@@ -599,8 +615,8 @@ class NamedTupleClassBuilder(abstract.PyTDClass):
 class NewType(abstract.PyTDFunction):
   """Implementation of typing.NewType as a function."""
 
-  def __init__(self, name, signatures, kind, vm):
-    super().__init__(name, signatures, kind, vm)
+  def __init__(self, name, signatures, kind, ctx):
+    super().__init__(name, signatures, kind, ctx)
     assert len(self.signatures) == 1, "NewType has more than one signature."
     signature = self.signatures[0].signature
     self._name_arg_name = signature.param_names[0]
@@ -614,7 +630,7 @@ class NewType(abstract.PyTDFunction):
     return val
 
   def call(self, node, func, args):
-    args = args.simplify(node, self.vm)
+    args = args.simplify(node, self.ctx)
     self.match_args(node, args, match_all_views=True)
     # As long as the types match we do not really care about the actual
     # class name. But, if we have a string literal value as the name arg,
@@ -623,7 +639,7 @@ class NewType(abstract.PyTDFunction):
     try:
       _ = abstract_utils.get_atomic_python_constant(name_arg, str)
     except abstract_utils.ConversionError:
-      name_arg = self.vm.convert.constant_to_var(
+      name_arg = self.ctx.convert.constant_to_var(
           f"_NewType_Internal_Class_Name_{self.internal_name_counter}_")
     type_arg = args.namedargs.get(self._type_arg_name) or args.posargs[1]
     try:
@@ -631,16 +647,17 @@ class NewType(abstract.PyTDFunction):
     except abstract_utils.ConversionError:
       # We need the type arg to be an atomic value. If not, we just
       # silently return unsolvable.
-      return node, self.vm.new_unsolvable(node)
+      return node, self.ctx.new_unsolvable(node)
     value_arg_name = "val"
     constructor = overlay_utils.make_method(
-        self.vm, node,
+        self.ctx,
+        node,
         name="__init__",
         params=[Param(value_arg_name, type_value)])
-    members = abstract.Dict(self.vm)
+    members = abstract.Dict(self.ctx)
     members.set_str_item(node, "__init__", constructor)
-    return self.vm.make_class(node, name_arg, (type_arg,),
-                              members.to_variable(node), None)
+    return self.ctx.vm.make_class(node, name_arg, (type_arg,),
+                                  members.to_variable(node), None)
 
 
 class Overload(abstract.PyTDFunction):
@@ -656,7 +673,7 @@ class Overload(abstract.PyTDFunction):
     for func in func_var.data:
       if isinstance(func, abstract.INTERPRETER_FUNCTION_TYPES):
         func.is_overload = True
-        self.vm.frame.overloads[func.name].append(func)
+        self.ctx.vm.frame.overloads[func.name].append(func)
 
     return node, func_var
 
@@ -673,11 +690,11 @@ class Optional(abstract.AnnotationClass):
   """Implementation of typing.Optional."""
 
   def _build_value(self, node, inner, ellipses):
-    self.vm.errorlog.invalid_ellipses(self.vm.frames, ellipses, self.name)
+    self.ctx.errorlog.invalid_ellipses(self.ctx.vm.frames, ellipses, self.name)
     if len(inner) != 1:
       error = "typing.Optional can only contain one type parameter"
-      self.vm.errorlog.invalid_annotation(self.vm.frames, self, error)
-    return abstract.Union((self.vm.convert.none_type,) + inner, self.vm)
+      self.ctx.errorlog.invalid_annotation(self.ctx.vm.frames, self, error)
+    return abstract.Union((self.ctx.convert.none_type,) + inner, self.ctx)
 
 
 class Literal(TypingContainer):
@@ -689,69 +706,69 @@ class Literal(TypingContainer):
     for i, param in enumerate(inner):
       # TODO(b/173742489): Once the enum overlay is enabled, we should
       # stop allowing unsolvable and handle enums here.
-      if (param == self.vm.convert.none or
+      if (param == self.ctx.convert.none or
           isinstance(param, abstract.LiteralClass) or
-          param == self.vm.convert.unsolvable and i not in ellipses):
+          param == self.ctx.convert.unsolvable and i not in ellipses):
         value = param
       elif (isinstance(param, abstract.ConcreteValue) and
             isinstance(param.pyval, (int, str, bytes))):
-        value = abstract.LiteralClass(param, self.vm)
+        value = abstract.LiteralClass(param, self.ctx)
       elif isinstance(param, abstract.Instance) and param.cls.is_enum:
-        value = abstract.LiteralClass(param, self.vm)
+        value = abstract.LiteralClass(param, self.ctx)
       else:
         if i in ellipses:
           invalid_param = "..."
         else:
           invalid_param = param.name
         errors.append((invalid_param, i))
-        value = self.vm.convert.unsolvable
+        value = self.ctx.convert.unsolvable
       values.append(value)
     if errors:
-      self.vm.errorlog.invalid_annotation(
-          self.vm.frames, self,
+      self.ctx.errorlog.invalid_annotation(
+          self.ctx.vm.frames, self,
           "\n".join("Bad parameter %r at index %d" % e for e in errors))
-    return self.vm.merge_values(values)
+    return self.ctx.convert.merge_values(values)
 
 
-def not_supported_yet(name, vm):
-  vm.errorlog.not_supported_yet(vm.frames, "typing." + name)
-  return vm.convert.unsolvable
+def not_supported_yet(name, ctx):
+  ctx.errorlog.not_supported_yet(ctx.vm.frames, "typing." + name)
+  return ctx.convert.unsolvable
 
 
-def build_namedtuple(vm):
-  if vm.python_version < (3, 6):
-    return NamedTupleFuncBuilder.make(vm)
+def build_namedtuple(ctx):
+  if ctx.python_version < (3, 6):
+    return NamedTupleFuncBuilder.make(ctx)
   else:
-    return NamedTupleClassBuilder(vm)
+    return NamedTupleClassBuilder(ctx)
 
 
-def build_newtype(vm):
-  return NewType.make("NewType", vm, "typing")
+def build_newtype(ctx):
+  return NewType.make("NewType", ctx, "typing")
 
 
-def build_noreturn(vm):
-  return vm.convert.no_return
+def build_noreturn(ctx):
+  return ctx.convert.no_return
 
 
-def build_overload(vm):
-  return Overload.make("overload", vm, "typing")
+def build_overload(ctx):
+  return Overload.make("overload", ctx, "typing")
 
 
-def build_typevar(vm):
-  return TypeVar.make("TypeVar", vm, "typing", pyval_name="_typevar_new")
+def build_typevar(ctx):
+  return TypeVar.make("TypeVar", ctx, "typing", pyval_name="_typevar_new")
 
 
-def build_typechecking(vm):
-  return vm.convert.true
+def build_typechecking(ctx):
+  return ctx.convert.true
 
 
-def build_cast(vm):
-  return Cast.make("cast", vm, "typing")
+def build_cast(ctx):
+  return Cast.make("cast", ctx, "typing")
 
 
-def build_final(vm):
-  vm.errorlog.not_supported_yet(vm.frames, "typing.final")
-  return vm.convert.name_to_value("typing.final")
+def build_final(ctx):
+  ctx.errorlog.not_supported_yet(ctx.vm.frames, "typing.final")
+  return ctx.convert.name_to_value("typing.final")
 
 
 typing_overlay = {
